@@ -57,6 +57,15 @@
 #ifndef GL_RGB8
 #define GL_RGB8 0x8051
 #endif
+
+static void makeCurrent(QGLContext *context)
+{
+    context->makeCurrent();
+
+    auto handle = context->contextHandle();
+    if (handle && QOpenGLContext::currentContext() != handle)
+        handle->makeCurrent(handle->surface());
+}
 #endif
 
 #include <QtDebug>
@@ -143,6 +152,11 @@ QAbstractVideoSurface::Error QVideoSurfaceGenericPainter::start(const QVideoSurf
 {
     m_frame = QVideoFrame();
     m_imageFormat = QVideoFrame::imageFormatFromPixelFormat(format.pixelFormat());
+    // Do not render into ARGB32 images using QPainter.
+    // Using QImage::Format_ARGB32_Premultiplied is significantly faster.
+    if (m_imageFormat == QImage::Format_ARGB32)
+        m_imageFormat = QImage::Format_ARGB32_Premultiplied;
+
     m_imageSize = format.frameSize();
     m_scanLineDirection = format.scanLineDirection();
     m_mirrored = format.property("mirrored").toBool();
@@ -191,10 +205,6 @@ QAbstractVideoSurface::Error QVideoSurfaceGenericPainter::paint(
                 m_imageSize.height(),
                 m_frame.bytesPerLine(),
                 m_imageFormat);
-        // Do not render into ARGB32 images using QPainter.
-        // Using QImage::Format_ARGB32_Premultiplied is significantly faster.
-        if (m_imageFormat == QImage::Format_ARGB32)
-            image = image.convertToFormat(QImage::Format_ARGB32_Premultiplied);
 
         const QTransform oldTransform = painter->transform();
         QTransform transform = oldTransform;
@@ -394,7 +404,7 @@ QAbstractVideoSurface::Error QVideoSurfaceGLPainter::setCurrentFrame(const QVide
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
     } else if (m_frame.map(QAbstractVideoBuffer::ReadOnly)) {
-        m_context->makeCurrent();
+        makeCurrent(m_context);
 
         for (int i = 0; i < m_textureCount; ++i) {
             glBindTexture(GL_TEXTURE_2D, m_textureIds[i]);
@@ -736,7 +746,7 @@ QAbstractVideoSurface::Error QVideoSurfaceArbFpPainter::start(const QVideoSurfac
 
     QAbstractVideoSurface::Error error = QAbstractVideoSurface::NoError;
 
-    m_context->makeCurrent();
+    makeCurrent(m_context);
 
     const char *program = 0;
 
@@ -861,7 +871,7 @@ QAbstractVideoSurface::Error QVideoSurfaceArbFpPainter::start(const QVideoSurfac
 void QVideoSurfaceArbFpPainter::stop()
 {
     if (m_context) {
-        m_context->makeCurrent();
+        makeCurrent(m_context);
 
         if (m_handleType != QAbstractVideoBuffer::GLTextureHandle)
             glDeleteTextures(m_textureCount, m_textureIds);
@@ -1114,7 +1124,7 @@ QAbstractVideoSurface::Error QVideoSurfaceGlslPainter::start(const QVideoSurface
 
     QAbstractVideoSurface::Error error = QAbstractVideoSurface::NoError;
 
-    m_context->makeCurrent();
+    makeCurrent(m_context);
 
     const char *fragmentProgram = 0;
 
@@ -1221,7 +1231,7 @@ QAbstractVideoSurface::Error QVideoSurfaceGlslPainter::start(const QVideoSurface
 void QVideoSurfaceGlslPainter::stop()
 {
     if (m_context) {
-        m_context->makeCurrent();
+        makeCurrent(m_context);
 
         if (m_handleType != QAbstractVideoBuffer::GLTextureHandle)
             glDeleteTextures(m_textureCount, m_textureIds);
@@ -1622,7 +1632,7 @@ void QPainterVideoSurface::setGLContext(QGLContext *context)
         //Set a dynamic property to access the OpenGL context
         this->setProperty("GLContext", QVariant::fromValue<QObject*>(m_glContext->contextHandle()));
 
-        m_glContext->makeCurrent();
+        makeCurrent(m_glContext);
 
         const QByteArray extensions(reinterpret_cast<const char *>(
                                         context->contextHandle()->functions()->glGetString(GL_EXTENSIONS)));
@@ -1733,13 +1743,13 @@ void QPainterVideoSurface::createPainter()
 #if !defined(QT_OPENGL_ES) && !defined(QT_OPENGL_DYNAMIC)
     case FragmentProgramShader:
         Q_ASSERT(m_glContext);
-        m_glContext->makeCurrent();
+        makeCurrent(m_glContext);
         m_painter = new QVideoSurfaceArbFpPainter(m_glContext);
         break;
 #endif // !QT_OPENGL_ES && !QT_OPENGL_DYNAMIC
     case GlslShader:
         Q_ASSERT(m_glContext);
-        m_glContext->makeCurrent();
+        makeCurrent(m_glContext);
         m_painter = new QVideoSurfaceGlslPainter(m_glContext);
         break;
     default:
